//
//  MyView.swift
//  Getting Started
//
//  Created by Nguyen Van Cuong on 11/5/17.
//  Copyright © 2017 Nguyen Van Cuong. All rights reserved.
//

import Macaw

class ShapeView: MacawView {
    required init?(coder aDecoder: NSCoder) {
        let shape = Rect(x: 375 / 2, y: 75, w: 175, h: 30).round(r: 5).fill(with: Color(val: 0xfcc07c))
        super.init(node: shape, coder: aDecoder)
    }
}

class TextView: MacawView {
    required init?(coder aDecoder: NSCoder) {
        let text = Text(text: "Sample",
                        font: Font(name: "Serif", size: 72),
                        fill: Color.blue)
        text.align = .mid
        text.baseline = .mid
        text.place = .move(dx: 100, dy: 100)
        super.init(node: text, coder: aDecoder)
    }
}

class GroupView: MacawView {
    required init?(coder aDecoder: NSCoder) {
        let shape = Shape(
            form: Rect(x: -100, y: -15, w: 200, h: 30).round(r: 5),
            fill: Color(val: 0xff9e4f))
        let text = Text(
            text: "Show",
            font: Font(name: "Serif", size: 21),
            fill: Color.white,
            align: .mid,
            baseline: .mid)
        let group = Group(contents: [shape, text], place: .move(dx: 375 / 2, dy: 75))
        super.init(node: group, coder: aDecoder)
    }
}

class GroupView2: MacawView {
    required init?(coder aDecoder: NSCoder) {
        
        let linearGradient = LinearGradient(degree: 90, from: Color(val: 0xfcc07c), to: Color(val: 0xfc7600))
        let shape = Shape(
            form: Rect(x: -100, y: -15, w: 200, h: 30).round(r: 5),
            fill: linearGradient)
        
        let text = Text(
            text: "Show",
            font: Font(name: "Serif", size: 21),
            fill: Color.white,
            align: .mid,
            baseline: .mid,
            opaque: false) //mark text should not receive any events
        
        let image = Image(
            src: "black_setting.png",
            w: 30, h: 30,
            place: .move(dx: -15 - 50, dy: -15),
            opaque: false)  //mark image should not receive any events
        
        shape.onTap { event in
            text.fill = Color.maroon // chỗ này hiện tại không hoạt động, cần remove linearGradient
        }
        
        let group = Group(contents: [shape, text, image], place: .move(dx: 375 / 2, dy: 75))
        super.init(node: group, coder: aDecoder)
    }
}

class ChartView: MacawView {
    required init?(coder aDecoder: NSCoder) {
        let button = ChartView.createButton()
        let chart = ChartView.createChart()
        button.onTap { _ in
            ChartView.bar?.opacityVar.animate(from: 0, to: 1)
        }
        super.init(node: Group(contents: [button, chart]), coder: aDecoder)
    }
    
    private static func createButton() -> Group {
        let shape = Shape(
            form: Rect(x: -100, y: -15, w: 200, h: 30).round(r: 5),
            fill: LinearGradient(degree: 90, from: Color(val: 0xfcc07c), to: Color(val: 0xfc7600)),
            stroke: Stroke(fill: Color(val: 0xff9e4f), width: 1))
        let text = Text(
            text: "Show", font: Font(name: "Serif", size: 21),
            fill: Color.white, align: .mid, baseline: .mid,
            place: .move(dx: 15, dy: 0), opaque: false)
        let image = Image(src: "black_setting.png", w: 30, place: .move(dx: -40, dy: -15), opaque: false)
        return Group(contents: [shape, text, image], place: .move(dx: 375 / 2, dy: 75))
    }
    
    private static func createChart() -> Group {
        var items: [Node] = []
        for i in 1...6 {
            let y = 200 - Double(i) * 30.0
            items.append(Line(x1: -5, y1: y, x2: 275, y2: y).stroke(fill: Color.green))
            items.append(Text(text: "\(i*30)", align: .max, baseline: .mid, place: .move(dx: -10, dy: y)))
        }
        items.append(Line(x1: 0, y1: 200, x2: 275, y2: 200).stroke(fill: Color.yellow))
        items.append(Line(x1: 0, y1: 0, x2: 0, y2: 200).stroke(fill: Color.yellow))
        items.append(createBars())
        return Group(contents: items, place: .move(dx: 50, dy: 200))
    }
    
    static let data: [Double] = [101, 142, 66, 178, 92]
    static let palette = [0xf08c00, 0xbf1a04, 0xffd505, 0x8fcc16, 0xd1aae3].map { val in Color(val: val)}
    static var bar: Group?
    private static func createBars() -> Group {
        var items: [Node] = []
        for (i, item) in data.enumerated() {
            let bar = Shape(
                form: Rect(x: Double(i) * 50 + 25, y: 0, w: 30, h: item),
                fill: LinearGradient(degree: 90, from: palette[i], to: palette[i].with(a: 0.3)),
                place: .move(dx: 0, dy: -data[i]))
            items.append(bar)
        }
        let group = Group(contents: items, place: .move(dx: 0, dy: 200))
        self.bar = group
        return group
    }
}

class SVGView: MacawView {
    required init?(coder aDecoder: NSCoder) {
        super.init(node: try! SVGParser.parse(path: "tiger"), coder: aDecoder)
    }
}
